@@ -1,10 +1,13 @@
 from typing import Tuple
 
-from torch import batch_norm, nn, ones, std_mean, zeros
+from torch import nn, ones, std_mean, zeros
+from torch.nn.functional import batch_norm
 
 
 class GANAdaptiveInstanceNorm(nn.Module):
     """
+    TODO NOT WORKING AS EXPECTED, PROBABLY NEEDS A REWRITE + INHERETING CLASSES
+
     Base class, should not be called directly.
 
     Custom Adaptive Instance Normalization from https://github.com/AdalbertoCq/Pathology-GAN/blob/master/models/generative/normalization.py (conditional_batch_norm)
@@ -20,7 +23,7 @@ class GANAdaptiveInstanceNorm(nn.Module):
         :type c: int
         :param spectral: defines if spectral normalization is used
         :type spectral: boolean
-        :param n_power_iterations: number of power iterations fpr spectral normalization
+        :param n_power_iterations: number of power iterations for spectral normalization
         :type n_power_iterations: int
         :param eps: hyperparameter for spectral normalization
         :type eps: float
@@ -28,7 +31,7 @@ class GANAdaptiveInstanceNorm(nn.Module):
         :type gain: float
         """
         super(GANAdaptiveInstanceNorm, self).__init__()
-        self.gain = gain  # orthogonal init
+        self.gain = gain
 
         def block_dense(in_feat, out_feat, activation=True, spectral=True, n_power_iterations=1, eps=1e-12):
             # Linear + Spectral Norm (optional) + Relu (optional)
@@ -43,7 +46,7 @@ class GANAdaptiveInstanceNorm(nn.Module):
 
             return layers
 
-        self.decay = 0.9
+        self.momentum = 0.9  # was decay in tf
         self.epsilon = 1e-5
 
         self.test_mean = zeros([inputs], requires_grad=False)
@@ -52,8 +55,9 @@ class GANAdaptiveInstanceNorm(nn.Module):
         # MLP for gamma, and beta
         inter_dim = int((inputs+c)/2)
 
+        # in_feat=c in tf code
         self.net = nn.Sequential(
-            *block_dense(in_feat=c, out_feat=inter_dim, activation=True, spectral=spectral, n_power_iterations=n_power_iterations, eps=eps))
+            *block_dense(in_feat=inputs, out_feat=inter_dim, activation=True, spectral=spectral, n_power_iterations=n_power_iterations, eps=eps))
 
         self.gamma = nn.Sequential(
             *block_dense(in_feat=inter_dim, out_feat=inputs, activation=True, spectral=spectral, n_power_iterations=n_power_iterations, eps=eps))
@@ -64,24 +68,29 @@ class GANAdaptiveInstanceNorm(nn.Module):
         self.init_weights()
 
     def forward(self, x):
-        # TODO check if split is required at all
-        # if  len(x) == 4:
-        # 	self.gamma = tf.expand_dims(tf.expand_dims(self.gamma, 1), 1)
-        # 	self.beta = tf.expand_dims(tf.expand_dims(self.beta, 1), 1)
-
         if len(x) == 4:
             batch_variance, batch_mean = std_mean(x, dim=[2, 3], keepdim=True)
         else:
             batch_variance, batch_mean = std_mean(x, dim=[1], keepdim=True)
 
-        batch_norm_output = batch_norm(
-            x, batch_mean, batch_variance, self.beta, self.gamma, self.epsilon)
+        x_net = self.net(x)
+
+        x_net_gamma = self.gamma(x_net)
+        x_net_beta = self.beta(x_net)
+
+        # according to https://discuss.pytorch.org/t/example-on-how-to-use-batch-norm/216/22, gamma and beta should correspond to .weight and .bias, respectively.
+        # batch_norm_output = batch_norm(
+        #     x, batch_mean, batch_variance, weight=x_net_gamma, bias=x_net_beta, momentum=self.momentum, eps=self.epsilon)
+
+        # workaround for the moment
+        batch_norm_output = (x_net_gamma*(x-batch_mean) /
+                             batch_variance)+x_net_beta
 
         return batch_norm_output
 
     def init_weights(self):
         for module in self.modules():
-            if isinstance(module, nn.Conv2d):
+            if isinstance(module, nn.Linear):
                 nn.init.orthogonal_(module.weight, gain=self.gain)
                 nn.init.constant_(module.bias, 0)
 
@@ -91,15 +100,15 @@ class GANAdaptiveInstanceNorm1d(GANAdaptiveInstanceNorm):
     Custom Adaptive Instance Normalization 1d Wrapper
     """
 
-    def __init__(self, feat: int, spectral: bool = False, n_power_iterations: int = 1, eps: float = 1e-12, gain: float = 1e-4):
+    def __init__(self, num_feat: int, spectral: bool = False, n_power_iterations: int = 1, eps: float = 1e-12, gain: float = 1e-4):
         """
         Init for Normalization1d
 
-        :param feat: number of input features
-        :type feat: int
+        :param num_feat: number of input features
+        :type num_feat: int
         :param spectral: defines if spectral normalization is used
         :type spectral: boolean
-        :param n_power_iterations: number of power iterations fpr spectral normalization
+        :param n_power_iterations: number of power iterations for spectral normalization
         :type n_power_iterations: int
         :param eps: hyperparameter for spectral normalization
         :type eps: float
@@ -108,7 +117,7 @@ class GANAdaptiveInstanceNorm1d(GANAdaptiveInstanceNorm):
         """
         # c is always 1 for 1d
         super(GANAdaptiveInstanceNorm1d, self).__init__(
-            inputs=feat, c=1, spectral=spectral, n_power_iterations=n_power_iterations, eps=eps, gain=gain)
+            inputs=num_feat, c=1, spectral=spectral, n_power_iterations=n_power_iterations, eps=eps, gain=gain)
 
 
 class GANAdaptiveInstanceNorm2d(GANAdaptiveInstanceNorm):
@@ -126,7 +135,7 @@ class GANAdaptiveInstanceNorm2d(GANAdaptiveInstanceNorm):
         :type c: int
         :param spectral: defines if spectral normalization is used
         :type spectral: boolean
-        :param n_power_iterations: number of power iterations fpr spectral normalization
+        :param n_power_iterations: number of power iterations for spectral normalization
         :type n_power_iterations: int
         :param eps: hyperparameter for spectral normalization
         :type eps: float
